<?php
require_once('op5/log.php');
/*
 * The rights are used as the intersection between the rights. Giving someone
 * full_access ∩ no_access results in no_access. Giving someone
 * by_contact_access ∩ no_edit results in being able to see, but not edit,
 * the objects the user is linked to by the contact access table.
 */
define('NACOMA_ACCESS_FULL_ACCESS', "allow");
define('NACOMA_ACCESS_VIEW_CONTACT', "view_contact");
define('NACOMA_ACCESS_EDIT_CONTACT', "edit_contact");
define('NACOMA_ACCESS_ADD_DELETE', "add_delete");
define('NACOMA_ACCESS_VIEW_ALL', "view_all");
define('NACOMA_ACCESS_EDIT_ALL', "edit_all");

/**
 * Returns an array of the access options as specified above.
 *
 */
function get_nacoma_access ($obj_type) {
	$log = Op5Log::instance('auth');
	$rights = get_rights();
	$user = get_user();

	switch ($obj_type) {
	 // put lots of rights in all lists
	 case 'user':
	 case 'group':
		return (isset($rights['access_rights']) && $rights['access_rights']) ? array(NACOMA_ACCESS_FULL_ACCESS) : array();
	 // and special-case what's in siiight
	 case 'graph_collection':
	 case 'graph_template':
	 case 'combined_graph':
	 case 'pnp':
		return (isset($rights['pnp']) && $rights['pnp']) ? array(NACOMA_ACCESS_FULL_ACCESS) : array();
	}

	// but it's the number of cases
	if (isset($rights[$obj_type])) {
		$log->log('debug', $user . ', Permissions for ' . $obj_type . ': allow');
		return array(NACOMA_ACCESS_FULL_ACCESS);
	}
	// that really drives you insa-a-a-a-ane
	/* oh crap, I got rid of this case - now my comments are lame */
	// let's do the rights dance agaaain
	else {
		$object_rights = array();
		$keys = array_keys($rights);
		foreach ($keys as $key) {
			if (preg_match('/^' . $obj_type . '[_]/', $key)) {
				$key = explode($obj_type . '_', $key);
				$key = 'NACOMA_ACCESS_' . strtoupper($key[1]);
				if (defined($key)) {
					$object_rights[] = constant($key);
				}
			}
		}
		$log->log('debug', $user . ', Permissions for ' . $obj_type . ': ' . implode(', ', $object_rights));
		return $object_rights;
	}
}

/**
 * Checks if the current user will be given by-contact-access access to an
 * object with the specified contacts and contact groups.
 *
 * Always returns true if the current right set is not by-contact-access.
 * Returns true if the user is listed in the contacts, or member of the listed
 * contact groups. Returns false otherwise.
 */
function is_user_contact($obj_type, $contacts, $contact_groups, $template) {
	$access = get_nacoma_access($obj_type);
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		return true;

	$username = get_user();
	$user = new contact_nacoma_Model($username);
	if ($template !== false) {
		$class = $obj_type.'_template_nacoma_Model';
		$template = new $class($template);
	}

	if (in_array($user->id, $contacts))
		return true;
	if (@in_array($user->id, $template->obj['contacts']))
		return true;
	if (isset($user->obj['contactgroups'])) {
		foreach($contact_groups as $group)
			if (in_array($group, $user->obj['contactgroups']))
				return true;
		if (@is_array($template->obj['contact_groups']))
			foreach ($template->obj['contact_groups'] as $group)
				if (in_array($group, $user->obj['contactgroups']))
					return true;
	}
	return false;
}
