<?php
/*
 * Copyright(C) 2004, 2005 op5 AB
 * All rights reserved.
 *
 */


/*****************************************************************************
 *                          Code section start                               *
 *****************************************************************************/

// show an error if nmap failed
function nmap_error($target_arg=false) {
	global $SUPPORT;

	echo "<p>Network Mapping Failed!</p>\n";
	echo "This usually happens beacuse of invalid target host specification (bad IP range). <br />
			Check your IP range ".($target_arg ? "(".$target_arg.")" : '')." and try again.<br /><br />";
	echo "Another reason could be when faulty permissions are set on the nmap executable.<br />\n";
	echo "You can usually solve this latter problem by logging in as root to your monitor server and issue the commands<pre>\n";
	echo "\n\tchown root:wheel /usr/bin/nmap\n\tchmod 4755 /usr/bin/nmap\n</pre>\n";
	echo "Should the problem persist, please contact $SUPPORT.<br />\n";
	html_end();
}

function parse_merlin_nodes($out) {
	// As soon as we have a number of pollers that's not completely insignificant,
	// they're almost certainly going to be arranged into groups of pollers.
	// Thus, caching the result for hostgroups we've already checked makes sense.
	$out_cache = array();
	$nodes = array('This server');
	$lines = explode("\n#", $out);
	foreach ($lines as $line) {
		if (!preg_match('/NAME=([^\n]+)/', $line, $matches)) {
			// first entry is empty, random stuff could happen
			continue;
		}
		$name = $matches[1];
		preg_match('/TYPE=([^\n]+)/', $line, $matches);
		$type = $matches[1];
		switch ($type) {
		 case 'poller':
			preg_match('/HOSTGROUP=([^\n]+)/', $line, $matches);
			$hostgroup = $matches[1];
			if (isset($out_cache[$hostgroup])) {
				if ($out_cache[$hostgroup])
					$nodes[] = $name;
				break;
			}
			$hg_obj = new hostgroup_nacoma_Model($hostgroup);
			if ($hg_obj->can_user_read($hg_obj->id)) {
				$nodes[] = $name;
				$out_cache[$hostgroup] = true;
			}
			else {
				$out_cache[$hostgroup] = false;
			}
			break;
		 case 'peer':
		 case 'master':
			$nodes[] = $name;
			break;
		 default:
			// not a peer, poller or master? lame, let's skip it
			break;
		}
	}
	return $nodes;
}

// Return a array of merlin nodes thats active
function get_merlin_nodes() {
	$sysret = probe("GET_MERLIN_NODES");
	return parse_merlin_nodes($sysret['buf']);
}

// run a pingsweep over a selected list of target ranges. Report only UP
function ping_sweep($target_arg) {
	$ip_range = explode(' ', $target_arg);
	$sysret = probe("PING_SWEEP", $ip_range);
	if($sysret['value'] !== 0) {
		nmap_error($target_arg);
	}

	$ret_list = array();
	$data = simplexml_load_string($sysret['buf']);
	foreach ($data->host as $host) {
		$hostname = false;
		if ($host->status['state'] != 'up')
			continue;
		if (isset($host->hostnames->hostname)) {
			foreach ($host->hostnames->hostname as $hname) {
				switch ($hname['type']) {
				 case 'PTR':
				 case 'user':
					$hostname = (string)$hname['name'];
					break;
				 default:
					break;
				}
			}
		}
		foreach ($host->address as $address) {
			switch ($address['addrtype']) {
			 case 'ipv4':
				$ret_list[(string)$address['addr']] = $hostname ? $hostname : (string)$address['addr'];
				break;
			 case 'ipv6':
			 default:
				break;
			}
		}
	}
	return($ret_list);
}

/* makes a pretty-list of checkboxes for services to be selected */
function make_checklist_checkboxes($service, $inp_name) {
	echo "<tr><td colspan=\"2\"><input type=\"checkbox\" op5_xtra=\"".$inp_name."\" id=\"{$inp_name}[all]\" class=\"check_all\"> &nbsp;<label for=\"{$inp_name}[all]\">Select all</label></td></tr>\n";
	foreach($service as $srv_base => $srv_array) {
		echo "  <tr class='header'><td colspan=\"2\">\n";
		echo "    " . strtoupper($srv_base) . "\n";
		echo "  </td></tr>\n";
		foreach($srv_array as $srv_name => $srv_desc) {
			echo "      <tr>\n";

			echo "        <td style=\"width: 10px;\">\n";
			echo "           <input type='checkbox' id='{$inp_name}[$srv_name]' name='{$inp_name}[$srv_name]'";
			// only PING is checked by default
			if($srv_desc === 'PING') echo " checked='checked'";
			echo ">\n      </td>\n";
			echo "			<td style=\"border-right: 1px solid #cdcddc\"><label for='{$inp_name}[$srv_name]'>$srv_desc</label></td>\n";
			echo "</tr>\n";
		}
	}
}

function get_chkcommands() {
	global $DATADIR;
    $chkcommands_file = 'chkcommands';
    $retval = 0;
    $output = array();

	$buf = read_file($DATADIR . $chkcommands_file);
	$lines = explode("\n", $buf);
	$cmd = array();
	foreach($lines as $line) {
		if(!strlen($line)) continue;
		$str = preg_split("/[\t]+/", $line);
		if (count($str) != 3)
			continue;
		$str[0] = trim($str[0]);
		$cmd[$str[0]]['service_description'] = trim($str[2]);
		$command = explode('!', trim($str[1]), 2);
		$cmd[$str[0]]['check_command'] = $command[0];
		$cmd[$str[0]]['check_command_args'] = isset($command[1])?$command[1]:'';
	}
	return $cmd;
}

/* wrapper for exec_cmd. scans $target and returns any open ports,
 * ordered neatly in an associative array */
function nmap_scan($target) {
	// do the scan
	$cmdret = probe("NMAP_SCAN", $target);
	if($cmdret['value'] !== 0) return(false);

	$net[0] = 'ping';

	$data = simplexml_load_string($cmdret['buf']);
	foreach ($data->host as $host) {
		$hostname = false;
		if (isset($host->ports) && isset($host->ports->port)) {
			foreach ($host->ports->port as $port) {
				if ((string)$port['protocol'] == 'tcp' && (string)$port->state['state'] == 'open')
					$net[(int)$port['portid']] = (string)$port->service['name'];
			}
		}
	}

	$ret['net'] = $net;
	return($ret);
}

function filter_monitored_hosts(&$target_list) {
	// remove those already configured
	$result = nacoma_db::exec_query('SELECT host_name, address FROM host ' .
				 'WHERE host_name != \'\'');

	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$ip = $row[1];
		$dns_name = $row[0];
		if(isset($target_list[$ip])) {
			unset($target_list[$ip]);
		}
		elseif(in_array($dns_name, $target_list)) {
			$tmp = array_search($dns_name, $target_list);
			unset($target_list[$tmp]);
		}
	}
        unset($result);
}

function do_snmp_scan($function, $args) {
	global $poller;
	if ($poller) {
		$out = probe('SNMP_SCAN', array('function' => $function, 'args' => $args));
		$out = preg_replace("/.*#--- REMOTE OUTPUT START ---\n*(.*)\n*#--- REMOTE OUTPUT DONE ---.*/ms", "$1", $out['buf']);
		return unserialize($out);
	}
	return call_user_func_array($function, $args);
}

function wmi_service_scan($address, $username, $password) {
	$ret = probe('WMI_SERVICE_SCAN', array('address' => $address, 'username' => $username, 'password' => $password));
	if ($ret['value'])
		return false;
	$ret = $ret['buf'];
	$ret = trim($ret);
	$svcs = explode("\n", $ret);
	array_shift($svcs);
	array_shift($svcs);
	$services = array();
	foreach ($svcs as $svc) {
		$svc = explode('|', $svc);
		$services[] = $svc[0];
	}
	return $services;
}

function nrpe_win_service_scan($address) {
	$max_left = 1000;
	$services = array();
	$options = array(
		'address' => $address,
		'arguments_allowed' => true,
		'start' => 0
	);
	$new_services = false;
	do {
		$ret = probe('WIN_SERVICE_SCAN', $options);
		if ($ret['value'] > 1 && $options['arguments_allowed']) {
			# The scan failed, but as NRPE args were used in the scan, that
			# could be why. Let's disable the use of args and give it another
			# shot. $new_services has still not been populated (still false),
			# so the while condition is still true at this point.
			$options['arguments_allowed'] = false;
			continue;
		}

		if (empty($ret['buf'])) {
			echo "No data returned from plugin\n";
			break;
		}

		$new_services = array();
		$result = explode(',', $ret['buf']);
		if (!empty($result)) {
			foreach ($result as $data) {
				$data = trim($data);
				if (trim($data === ''))
					continue;
				$new_services[$data] = trim($data);
			}
		}
		$new_keys = array_keys($new_services);
		$keys = array_keys($services);
		if (!empty($keys) && $keys[0] == $new_keys[0]) {
			# we are always gettings the same result back probably since
			# NRPE allows arguments but external script doesn't
			echo "<b>It seems that you need to allow arguments for external scripts in NSCLIENT.<br />" .
				"Either set all allowed_arguments=1 or remove them completely.<br />" .
				"As things are now - you will only get the first 20 services.</b>";
			break;
		}
		$services = array_merge($services, $new_services);
		$max_left -= count($new_services);
		$options['start'] += count($new_services);
	} while ($max_left > 0 && ($new_services === false || count($new_services) === 20));

	if (empty($services) && $max_left === 1000) {
		echo "No service list returned from host.\n";
		return false;
	}

	return $services;
}
