<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

$base_path = dirname(__FILE__);
$cli = PHP_SAPI == 'cli';
// starts session:
require_once($base_path.'/include/config.inc.php');
if ($cli)
	give_infinite_rights();
require_once($base_path.'/classes/common_gui.php');

$action = '';
$name = '';
$force = false;
$mgmt_pack_dir = "/usr/share/monitor/management-packs";

function usage($exit_code)
{
	global $argv;
	print "{$argv[0]} <--import|--export> [--force] --name <name>\n" .
		"{$argv[0]} [--list]\n";
	exit($exit_code);
}

function flash_and_redirect($msg, $success = false) {
	$_SESSION['nacoma_man_pack_flash'] = $msg;
	header("Location: ".$_SERVER['PHP_SELF']);
	exit(!(int)(bool)$success);
}

function flash() {
	if(isset($_SESSION['nacoma_man_pack_flash'])) {
		$msg = $_SESSION['nacoma_man_pack_flash'];
		unset($_SESSION['nacoma_man_pack_flash']);
		return "<p><strong>$msg</strong></p>";
	}
	return null;
}

if ($cli) {
	for ($i = 1; $i < count($argv); $i++) {
		switch($argv[$i]) {
		 case '--import':
			$action = 'import';
			break;
		 case '--export':
			$action = 'export';
			break;
		 case '--list':
			$action = 'list';
			break;
		 case '--force':
			$force = true;
			break;
		 case '--name':
			if ($i + 1 < count($argv)) {
				$name = $argv[++$i];
				break;
			}
			usage(1);
			break;
		 case '--help':
		 case '-h':
			usage(0);
			break;
		 default:
			usage(1);
			break;
		}
	}
}
else {
	if(isset($_FILES) && $_FILES) {
		if($_FILES['management_pack']['size'] == 0) {
			flash_and_redirect("You need to choose a JSON file to upload");
		}
		if($_FILES['management_pack']['size'] > 10485760) {
			flash_and_redirect("You probably should not upload management packs larger than 10 MB");
		}
		if($_FILES['management_pack']['error'] != UPLOAD_ERR_OK) {
			flash_and_redirect("Could not upload management pack: ".$_FILES['management_pack']['error']);
		}
		$tmp_file = $_FILES['management_pack']['tmp_name'];
		$data = json_decode(file_get_contents($tmp_file), true);
		if($data === null) {
			flash_and_redirect("Invalid JSON, check the file format and try again");
		}
		try {
			management_pack_management_nacoma_Model::validate_input($data);
		} catch (ManagementPackManagementValidationException $ex) {
			flash_and_redirect($ex->getMessage());
		}
		$force = false;
		if(isset($_POST['force'])) {
			$force = (bool) $_POST['force'];
		}
		$man_pack = new management_pack_management_nacoma_Model();
		$pack = $data['management_pack'];
		switch (($ret = $man_pack->import($pack, $force))) {
			case true:
				flash_and_redirect("Uploaded management pack ".$pack['management_pack']['management_pack_name'], true);
			case false:
				flash_and_redirect("Cannot import management pack '".$pack['management_pack']['management_pack_name']."', because it has been modified. You can try to force the import, but that might cause checks to change or fail.");
			default:
				flash_and_redirect(implode("<br>", $ret->validation_error_list()));
		}
	} elseif (!isset($_GET['action'])) {
		$action = 'list';
	} else {
		if (isset($_GET['management_pack_name']))
			$name = $_GET['management_pack_name'];
		else
			bug("Missing object name", true);

		if ($_GET['action'] == 'import')
			$action = 'import';
		else if ($_GET['action'] == 'export')
			$action = 'export';
		else
			bug("Invalid action", true);

		$force = isset($_GET['force']);
	}
}

if ($action == 'export') {
	$pack = new management_pack_management_nacoma_Model();
	$ary = $pack->export($name, 'This is a management pack');
	if (is_array($ary)) {
		header('Content-Type: application/json');
		header('Content-Disposition: attachment; filename="'.$_GET['management_pack_name'].'.json"');
		print json_encode($ary);
		exit(0);
	} else {
		bug("Error exporting the management pack: $ary", true);
	}
}

$page->html_start("Management pack manager");
nacoma_objlock::die_if_locked_hard();

if (!in_array(NACOMA_ACCESS_EDIT_ALL, get_nacoma_access('management_pack'))) {
	deny_access("You are not allowed to use this feature because you haven't got access to edit management packs.");
}

echo flash();

$page->import_if_changes();

if ($action == 'import') {
	$mgmt_file = $mgmt_pack_dir."/".$name.".json";
	if($mgmt_file != realpath($mgmt_file)) {
		print ($cli?'':'<p>')."Could not find the correct file $name.json to import, does it exist in $mgmt_pack_dir?".($cli?'':'</p>');
		$page->html_end(1);
	}
	$data = json_decode(file_get_contents($mgmt_file), true);
	if($data === null) {
		print ($cli?'':'<p>')."File doesn't seem to contain valid JSON (".json_last_error().")".($cli?'':'</p>');
		$page->html_end(1);
	}
	try {
		management_pack_management_nacoma_Model::validate_input($data);
	} catch(Exception $e) {
		print $e->getMessage();
		$page->html_end(1);
	}
	$mgmt_pack = $data['management_pack'];
	$pack = new management_pack_management_nacoma_Model();
	$ret = $pack->import($mgmt_pack, $force);
	if (is_array($ret)) {
		$page->print_validation_error_table($ret);
		$page->html_end(1);
	} else if ($ret) {
		if (!$cli)
			print "<p><img src=\"images/icons/shield-ok_16x16.png\" alt=\"\" style=\"margin-bottom: -3px\" /> <strong>Successfully imported '<a href=\"edit.php?obj_type=management_pack&obj_id=$ret\">{$mgmt_pack['management_pack']['management_pack_name']}'</a></strong></p>";
		else
			$page->html_end(0);
	} else {
		$text = "<p><img src=\"images/icons/shield-critical_16x16.png\" style=\"margin-bottom: -3px\" /><strong>Cannot import management pack '".$mgmt_pack['management_pack']['management_pack_name']."', because it has been modified.</strong> You can try to force the import, but that might cause checks to change or fail.</p>\n";
		$res = $pack->is_imported($mgmt_pack);
		if (!is_array($res)) {
			$text .= "<p>Additionally, it wasn't possible to retrieve information about why this failed. Please contact op5 support.</p>";
		} else {
			$text .= "<p>Errors:</p>\n<ul><li>".implode("</li>\n<li>", $res)."</li></ul>\n";
		}
		if ($cli) {
			print strip_tags($text);
			$page->html_end(1);
		}
		else {
			print $text;
		}
	}
}

$mgmt_packs = glob($mgmt_pack_dir."/*.json");

if (!$cli) {
?>
<p>This tool helps you activate and transfer management packs.</p>
<fieldset class="toc">
<legend><b>Contents</b></legend>
<ul>
	<li><a href="#import">Import</a><ul>
		<li><a href="#upload">Upload management pack</a></li>
		<li><a href="#activate">Activate standard packs</a></li>
	</ul></li>
	<li><a href="#export">Export</a></li>
</ul>
</fieldset>

</div>
<?php
	$page->print_related('management_pack_management', array(
		'<img src="images/icons/management-packs.png" alt="" /> <a href="edit.php?obj_type=management_pack">Management packs</a>',
		'<img src="images/icons/host-groups.png" alt="" /> <a href="edit.php?obj_type=hostgroup">Hostgroups</a>',
		'<img src="images/icons/commands.png" alt="" /> <a href="metadata.php">Manage check commands</a>'
	));
?>
<div id="object-table">
<div id="import" style="margin-top: -30px; margin-bottom: 30px;"></div>
<h2>Import</h2>
<p class="max">There are two ways to get new and updated management packs onto your Monitor system. You can either upload pack files from your computer, or you can activate packs or pack upgrades that were already delivered with your system.<p>
<div class="magic max">
<div id="upload" style="margin-top: -30px; margin-bottom: 30px;"></div>
<h3>Upload management pack</h3>
<p class="max">You can import a management pack by uploading a json file that you have previously exported.</p>
<p class="max">By forcing the import, you will disregard conflicts with existing checks, services or hostgroups and simply replace them. This might cause incorrect configuration to be used. Use with caution.</p>
</div>
<form enctype="multipart/form-data" action="<?php echo $_SERVER['PHP_SELF'] ?>" method="post">
	<p><input name="management_pack" type="file" /> <label><input type="checkbox" name="force" value="1" /> Force import?</label></p>
	<p><input type="submit" value="Upload json file" /></p>
</form>
<div class="magic max">
<div id="activate" style="margin-top: -30px; margin-bottom: 30px;"></div>
<h3>Activate standard packs</h3>
<p class="max">These packs are shipped with op5 Monitor, but for some reason they could not be automatically imported. There might be conflicts between them and your configuration that needs to be handled manually, or there might be new versions with new checks that might require your attention.</p>
</div>
</div>
<?php
} else {
	print "Import\n\n";
}

if($mgmt_packs) {
$rendered = 0;
foreach ($mgmt_packs as $mgmt_pack) {
	$data = json_decode(file_get_contents($mgmt_pack), true);
	if($data === null) {
		print "$mgmt_pack doesn't seem to contain valid JSON (".json_last_error().")";
		continue;
	}
	$pack = new management_pack_management_nacoma_Model();
	try {
		management_pack_management_nacoma_Model::validate_input($data);
	} catch(ManagementPackManagementValidationException $e) {
		print "JSON file $mgmt_pack doesn't seem to contain valid management pack data: ".$e->getMessage();
		continue;
	}
	$saved_mgmt = new management_pack_nacoma_Model($data['management_pack']['management_pack']['management_pack_name']);
	if ($data['changes'] <= $saved_mgmt->obj['version'])
		continue;
	$imported = $pack->is_imported($data['management_pack']);
	if ($imported === 1)
		continue;
	$rendered++;
	if (!$cli) {
?>
<table class="max magic zebra ObjTable">
	<tr>
		<td style="padding: 0; border: 0;">
			<table class="max header">
				<tr>
					<td class="CmdBar"><?php echo $data['management_pack']['management_pack']['management_pack_name']; ?></td>
					<td class="CmdBarItem"><a href="mgmtpackimport.php?action=import&amp;management_pack_name=<?php echo $data['management_pack']['management_pack']['management_pack_name'] ?>">Activate</a></td>
				</tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>
			<p class="max"><?php echo $data['management_pack']['management_pack']['description'] ?></p>
			<b>Changes</b>
			<ul>
<?php
		foreach ($data['changes'] as $index => $change) {
			$your_version = "";
			if ($change['version'] == $saved_mgmt->obj['version']) {
				if($change['version'] < count($data['changes'])) {
					// show currently installed version if it's
					$your_version = " <strong>(currently installed version)</strong>";
				} else {
					$your_version = " <strong>(currently installed version, but conflicts exist)</strong>";
				}
			}
			print "<li>{$change['version']}) {$change['description']}$your_version</li>";
		}
?>
			</ul>
<?php
		if (is_array($imported)) {
			$msg = '<ul><li>'.implode('</li><li>', $imported).'</li></ul>';
			?>
			<p><b>Warning:</b> This pack was found modified from what was expected:</p>
			<?php echo $msg; ?>
			<p class="max">You can force the activation of the pack, which will overwrite the unexpected differences. This might have unexpected effects on your configuration.</p>
			<div class="magic">
			<form action="<?php echo $_SERVER['PHP_SELF']; ?>" method="GET">
				<input type="submit" name="x" value="Force activate"/>
				<input type="hidden" name="action" value="import"/>
				<input type="hidden" name="management_pack_name" value="<?php echo $data['management_pack']['management_pack']['management_pack_name'] ?>"/>
				<input type="hidden" name="force" value="1"/>
			</form>
			</div>
			<?php
		}
	} else {
		print $data['management_pack']['management_pack']['management_pack_name']."\n";
		print $data['management_pack']['management_pack']['description']."\n";
		if (is_array($imported))
			print "Warning: This pack appears partially imported:\n".implode("\n", $imported)."\nRe-importing it can have unforeseen effects on your existing configuration.";
	}
	if (!$cli) {
?>
		</td>
	</tr>
</table>
<?php
	}
}
}
if (!$cli && (!$mgmt_packs || $rendered == 0)) {
	echo '<p>Found no changed management packs to activate.</p>';
}

if (!$cli) {
?>
<div class="magic max">
	<div id="export" style="margin-top: -30px; margin-bottom: 30px;"></div>
	<h2>Export</h2>
	<p class="max">You can exchange these management packs between Monitor installations by "exporting" them. You will get a json file with all objects that the management pack needs, and can simply upload it to any other Monitor installation.<br/><b>Note:</b> if you need custom check plugins, those must be transfered separately.<br/><b>Note:</b> for portability reasons, relations to other objects, such as Contacts and Timeperiods, will not be included.</p>
</div>
<?php
} else {
	print "Export";
}
$res = nacoma_db::exec_query('SELECT management_pack_name, description FROM management_pack INNER JOIN hostgroup ON hostgroup_name = hostgroup');
while ($row = $res->fetch(PDO::FETCH_ASSOC)) {
	if (!$cli) {
?>
<table class="max magic zebra ObjTable">
	<tr>
		<td style="padding: 0; border: 0;">
			<table class="max header">
				<tr>
					<td class="CmdBar"><?php echo $row['management_pack_name']; ?></td>
					<td class="CmdBarItem"><a href="mgmtpackimport.php?action=export&amp;management_pack_name=<?php echo $row['management_pack_name'] ?>">Export</a></td>
				</tr>
			</table>
		</td>
	</tr>
	<tr>
		<td>
			<p class="max"><?php echo $row['description'] ?></p>
		</td>
	</tr>
</table>
<?php
	} else {
		print $row['management_pack_name']."\n";
		print $row['description']."\n";
	}
}

if(!$cli) {
?>
</div>
</div>
</body>
</html>
<?php }
