<?php

/**
 * A host group definition is used to group one or more hosts together for
 * simplifying configuration.
 */
class hostgroup_nacoma_Model extends naemon_nacoma_Model
{
	/** object type */
	const TYPE = 'hostgroup';
	var $slave_types = array(
		'service',
		'hostdependency',
		'hostescalation',
	);

	public $properties = array (
		'topic' => 'hostgroup_name',
		'var_index' => array (
			'hostgroup_name' => 'string',
			'alias' => 'string',
			'members' => 'mselect',
			'hostgroup_members' => 'mselect',
			'notes' => 'string',
			'notes_url' => 'notesurl',
			'action_url' => 'string',
			'file_id' => 'select',
			'register' => 'bool'
		),
		'widget' => array (
			'register' => 'hidden'
		),
		'required' => array (
			'hostgroup_name',
			'file_id'
		),
		'default' => array (
			'file_id' => 'etc/hostgroups.cfg'
		),
		'special' => array (
			'file_id' => 'FILE',
			'members' => 'host',
			'hostgroup_members' => 'hostgroup'
		)
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.hostgroups";
	}

	protected function get_reasons_to_not_delete(&$all, &$hard)
	{
		parent::get_reasons_to_not_delete($all, $hard);

		$res = nacoma_db::exec_query('SELECT id, management_pack_name FROM management_pack WHERE hostgroup = '.nacoma_db::quote_string($this->obj[$this->properties['topic']]));
		foreach ($res as $row) {
			if (!isset($hard['management_pack']))
				$hard['management_pack'] = array();
			$hard['management_pack'][$row[0]] = "This hostgroup is in use by management pack \"{$row[1]}\"";
		}
		if (isset($hard['management_pack']))
			$all['management_pack'] = $hard['management_pack'];
		unset($res);
	}

	public function save_object($user=true)
	{
		$pre_name = get_object_name_by_id(static::TYPE, $this->id);
		$post_name = $this->get_object_name();
		if (!parent::save_object($user))
			return false;
		if ($pre_name == $post_name)
			return true;
		nacoma_db::exec_query('UPDATE management_pack SET hostgroup = '.nacoma_db::quote_string($post_name).' WHERE hostgroup = '.nacoma_db::quote_string($pre_name));
		return true;
	}
}
