<?php

require_once('op5/config.php');
require_once('op5/log.php');
require_once('op5/auth/Authorization.php');

/**
 * Users can log onto the Monitor GUI and see the objects that
 * they are configured to see. This differs from the concept of
 * contacts, who doesn't necessarily have access to the GUI.
 */
class user_nacoma_Model extends object_nacoma_Model {
	public $lazy_write = false; /**< If true, don't automatically generate auth_users.yml - performance option when handling many users */
	public $properties = array(
		'var_index' => array(
			'username'		=> 'string',
			'realname'		=> 'string',
			'password'		=> 'password',
			'password_algo' => 'norender',
			'modules'		=> 'mselect',
			'groups'		=> 'mselect',
			'Access Levels' => 'norender' // deprecated, only for API compatibility purpouses
		),
		'widget' => array(
			'password_algo' => 'norender',
		),
		'default' => array(
			'realname'		=> '',
			'password'		=> '',
			'modules'		=> '',
			'groups'		=> '',
			'password_algo' => '',
		),
		'visible' => array(
			'username',
			'realname',
			'password',
			'modules',
			'groups'
		),
		'special' => array(
			'modules'		=> array(),
			'groups'		=> array()
		),
		'required' => array(),
		'label' => array(
			'realname'		=> 'Real name',
			'password'		=> 'Password<br>Repeat',
			'modules'		=> 'Auth Modules',
			'groups'		=> 'Group Membership'
			),
		'topic' => 'username',
		'deprecated_names' => array(
			'Username' => 'username',
			'Realname' => 'realname',
			'Password' => 'password',
			'Role' => 'groups'
		)
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.users";
	}

	public function get_properties() {
		self::get_groups();
		$this->properties['special']['groups'] = (array)array_combine(array_keys(self::get_groups()), array_keys(self::get_groups()));
		$this->properties['special']['modules'] = (array)array_combine(self::$auth_modules, self::$auth_modules);
		return $this->properties;
	}

	/** type name */
	const TYPE = 'user';
	/** doesn't actually exist, but it's the correct join pre-/suffix */
	public $db_table = 'username';

	private static $users = false;
	private static $groups = false;
	private static $auth_modules = false;
	private static $auth_module_requires_password = false;

	/* do NOT use this outside tests */
	public static function drop_cache()
	{
		self::$users = false;
		self::$groups = false;
	}

	private static function get_groups() {
		if(is_array(self::$groups)) {
			return self::$groups;
		}
		return self::$groups = Op5Config::instance()->getConfig('auth_groups');
	}

	public static function get_auth_modules_metadata($field = false) {
		return op5auth::instance()->get_metadata($field);
	}

	private static function read_users()
	{
		self::get_groups();
		self::$auth_modules = self::get_auth_modules_metadata('require_user_configuration');
		self::$auth_module_requires_password = self::get_auth_modules_metadata('require_user_password_configuration');

		if (count(self::$auth_module_requires_password) < 1) {
			self::$auth_module_requires_password = false;
		}

		if(is_array(self::$users)) {
			return;
		}

		$auth_users = Op5Config::instance()->getConfig('auth_users');

		self::$users = array();
		foreach($auth_users as $username => $userdata) {
			self::$users[$username] = $userdata;
		}
	}

	/**
	 * Read the rights and password files and statically store the data therein
	 */
	public static function get_users() {
		self::read_users();

		$user_objs = array();
		foreach(self::$users as $user => $userdata) {
			$obj = new self($user);
			$user_objs[] = $obj;
		}

		return $user_objs;
	}

	/**
	 * @param $username
	 * @return user_nacoma_Model|false
	 */
	public function get_object_by_name($username) {
		self::read_users();

		$this->obj['username'] = $username;
		if (!isset(self::$users[$username])) {
			return false;
		}
		$this->id = $username;

		$this->obj = $this->get_default_object();
		if ($username !== false && $username !== 'new') {
			foreach(self::$users[$username] as $key => $value) {
				$this->obj[$key] = $value;
			}
			$this->properties['widget']['username'] = 'hidden';
		} else {
			$this->obj['username'] = '';
		}

		/* Never push password or password hash to gui, it won't be updated if not explicitly updated */
		$this->obj['password'] = '';
		$this->obj['password_algo'] = '';

		return $this->obj;
	}

	public function __construct($id = false) {
		self::read_users();
		$this->get_properties();
		$this->type = static::TYPE;

		if (isset(self::$users[$id])) {
			$this->get_object_by_name($id);
		}
	}

	public function get_object_name() {
		return (isset($this->obj['username']) && $this->obj['username']) ? $this->obj['username'] : 'New User';
	}

	/**
	 * Write user files to disk
	 *
	 * If you set $lazy_write to true, this must be manually called, otherwise
	 * it will be called automatically at the end of relevant methods
	 *
	 * Depends on self::read_users(), which should be solved in
	 * self::__construct()
	 */
	public static function write_users() {
		return Op5Config::instance()->setConfig('auth_users', self::$users);
	}

	/**
	 * Set user password
	 *
	 * @param $password
	 */
	public function set_password($password) {
		/* TODO: Which hashing algorithm? crypt is the only simple one available which has salt... */
		$this->obj['password'] = crypt($password);
		$this->obj['password_algo'] = 'crypt';
	}

	/**
	 * @param $user
	 * @return boolean
	 */
	public function save_object($user=true) {
		$log = Op5Log::instance('auth');
		self::get_groups(); /* For validation */

		if(!$this->validate_object()) {
			return false;
		}

		if (!empty($this->obj['password'])) {
			$this->set_password($this->obj['password']);
		} elseif(isset(self::$users[$this->obj['username']])) {
			/* If no password update, keep old... */
			if(isset(self::$users[$this->obj['username']]['password'])) {
				$this->obj['password'] = self::$users[$this->obj['username']]['password'];
			}
			if(isset(self::$users[$this->obj['username']]['password_algo'])) {
				$this->obj['password_algo'] = self::$users[$this->obj['username']]['password_algo'];
			}
		}
		if (isset($this->obj['Access Levels'])) {
			$user_group = new usergroup_nacoma_Model();
			$user_group->obj['name'] = 'user_'.$this->obj['username'];
			$user_group->obj['access_rights'] = Op5Authorization::nagios_rights_to_op5auth($this->obj['Access Levels']);
			if (!$user_group->save_object()) {
				$this->validation_error('Access Levels', "Couldn't create group from levels");
				$this->validation_error_list = array_merge($this->validation_error_list, $user_group->validation_error_list);
				return false;
			}
			if (!isset($this->obj['groups']))
				$this->obj['groups'] = array();
			array_unshift($this->obj['groups'], $user_group->id);
			unset($this->obj['Access Levels']);
		}

		if (empty($this->validation_error_list)) {
			if(empty($this->obj['username'])) {
				return false;
			}

			self::$users[$this->obj['username']] = $this->obj;

			if (!$this->lazy_write)
				self::write_users();
			return true;
		}
		return false;
	}

	function validate_object() {
		if (!isset($this->obj['modules'])) {
			$this->obj['modules'] = array();
		}
		foreach ($this->obj as $k => $v) {
			$v_type = $this->properties['var_index'][$k];
			# This is all wrong, is it not?
			try {
				// ucfirst = make sure it's loaded as a library
				$class = ucfirst($v_type) . '_Nacomawidget';
				$class = new $class();
				$this->obj[$k] = $class->validate($this, $k, $this->obj[$k]);
			}
			catch (widget_validation_exception $ex) {
				if ($k === 'password') {
					if (self::$auth_module_requires_password !== false) {
						if (empty($v[1]) && empty($v[2])) {
							if (count(array_intersect(self::$auth_module_requires_password, $this->obj['modules'])) > 0 &&
								!isset(self::$users[$this->obj['username']]['password'])) {
								/**
								 * Do not allow a previously created user that adds a module that requires passwords
								 * to leave the password fields blank if a password is not previously set
								 */
								return $this->validation_error("password", "You must set a password for user: " . $this->obj['username']);
							}
							// New user that added a module that doesn't require passwords or existing user
							$this->obj['password'] = null;
							continue;
						}
					} else {
						// No auth modules needs passwords
						$this->obj['password'] = null;
						continue;
					}
				}
				return $this->validation_error($k, $ex->getMessage());
			}
		}

		if ($this->id == 'new' || !$this->id) {
			if (empty($this->obj['username'])) { // new user without username is no new user
				return true;
			}
			if (isset(self::$users[$this->obj['username']])) {
				return $this->validation_error('username', "A user named {$this->obj['username']} already exists. Not re-added.");
			}
			if (self::$auth_module_requires_password !== false &&
				count(array_intersect(self::$auth_module_requires_password, $this->obj['modules'])) > 0 &&
				empty($this->obj['password'])) {
					return $this->validation_error('password', 'Password for new user empty. Not adding.');
			}
		}

		return true;
	}

	public function delete_object()
	{
		$this->banish_object();
	}

	public function banish_object($user=true)
	{
		$username = $this->get_object_name();

		unset(self::$users[$username]);

		if (!$this->lazy_write) {
			self::write_users();
		}
		return true;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = array();
		if ($this->id !== 'new' && !empty($this->id))
			$cmd_links['delete'] = "<a title='Delete this object' class='delete' href='access?action=delete&amp;delete=".html::specialchars($this->id)."'>";
		return $cmd_links;
	}

	public static function get_objects_for_user($write = false)
	{
		self::read_users();

		$usernames = array();
		foreach(self::$users as $user => $level) {
			$usernames[] = $user;
		}
		return $usernames;
	}

	public function normalize($obj = false, $abort_on_error=true)
	{
		if ($obj === false)
			$obj = $this->obj;

		if (isset($obj['Role'])) {
			if (!isset($obj['groups']))
				$obj['groups'] = array();
			array_unshift($obj['groups'], $obj['Role']);
			unset($obj['Role']);
		}
		parent::normalize($obj, $abort_on_error);
	}

	public function denormalize($obj = false) {
		$obj = parent::denormalize($obj);

		# Warning! Terrible magic here!
		# Password is never a meaningful value to show deserialized,
		# because it's always encrypted and secret, so unset it.
		unset($obj['password']);

		if (isset($obj['Roles']))
			$obj['Roles'] = implode(',', $obj['Roles']);
		$obj['Access Levels'] = array();

		return $obj;
	}
}
